@@ -350,7 +350,18 @@ bool cf_load_file(const struct CfContext *cf, const char *fn)
 bool cf_set_int(struct CfValue *cv, const char *value)
 {
 	int *ptr = cv->value_p;
-	*ptr = atoi(value);
+	char *end;
+	long val;
+
+	errno = 0;
+	val = strtol(value, &end, 0);
+	if (end == value || *end != 0) {
+		/* reject partial parse */
+		if (!errno)
+			errno = EINVAL;
+		return false;
+	}
+	*ptr = val;
 	return true;
 }
 
