@@ -23,11 +23,12 @@
 #include "bouncer.h"
 
 #include <usual/regex.h>
+#include <usual/netdb.h>
 
 /* regex elements */
 #define WS0	"[ \t\n\r]*"
 #define WS1	"[ \t\n\r]+"
-#define WORD	"([0-9a-z_]+)"
+#define WORD	"(\"([^\"]+|\"\")*\"|[0-9a-z_]+)"
 #define STRING	"'(([^']*|'')*)'"
 
 /* possible max + 1 */
@@ -35,9 +36,9 @@
 
 /* group numbers */
 #define CMD_NAME 1
-#define CMD_ARG 3
+#define CMD_ARG 4
 #define SET_KEY 1
-#define SET_VAL 3
+#define SET_VAL 4
 
 typedef bool (*cmd_func_t)(PgSocket *admin, const char *arg);
 struct cmd_lookup {
@@ -141,7 +142,7 @@ bool admin_ready(PgSocket *admin, const char *desc)
 	pktbuf_static(&buf, tmp, sizeof(tmp));
 	pktbuf_write_CommandComplete(&buf, desc);
 	pktbuf_write_ReadyForQuery(&buf);
-	return pktbuf_send_immidiate(&buf, admin);
+	return pktbuf_send_immediate(&buf, admin);
 }
 
 /*
@@ -468,6 +469,14 @@ static bool admin_show_lists(PgSocket *admin, const char *arg)
 	SENDLIST("login_clients", statlist_count(&login_client_list));
 	SENDLIST("free_servers", slab_free_count(server_cache));
 	SENDLIST("used_servers", slab_active_count(server_cache));
+	{
+		int names, zones, qry, pend;
+		adns_info(adns, &names, &zones, &qry, &pend);
+		SENDLIST("dns_names", names);
+		SENDLIST("dns_zones", zones);
+		SENDLIST("dns_queries", qry);
+		SENDLIST("dns_pending", pend);
+	}
 	admin_flush(admin, buf, "SHOW");
 	return true;
 }
@@ -515,7 +524,7 @@ static void socket_row(PktBuf *buf, PgSocket *sk, const char *state, bool debug)
 {
 	int pkt_avail = 0, send_avail = 0;
 	char ptrbuf[128], linkbuf[128];
-	char l_addr[32], r_addr[32];
+	char l_addr[PGADDR_BUF], r_addr[PGADDR_BUF];
 	IOBuf *io = sk->sbuf.io;
 
 	if (io) {
@@ -748,6 +757,67 @@ static bool admin_show_mem(PgSocket *admin, const char *arg)
 	return true;
 }
 
+/* Command: SHOW DNS_HOSTS */
+
+static void dns_name_cb(void *arg, const char *name, const struct addrinfo *ai, usec_t ttl)
+{
+	PktBuf *buf = arg;
+	char *s, *end;
+	char adrs[1024];
+	usec_t now = get_cached_time();
+
+	end = adrs + sizeof(adrs) - 2;
+	for (s = adrs; ai && s < end; ai = ai->ai_next) {
+		if (s != adrs)
+			*s++ = ',';
+		sa2str(ai->ai_addr, s, end - s);
+		s += strlen(s);
+	}
+	*s = 0;
+
+	pktbuf_write_DataRow(buf, "sqs", name, (ttl - now) / USEC, adrs);
+}
+
+static bool admin_show_dns_hosts(PgSocket *admin, const char *arg)
+{
+	PktBuf *buf;
+
+	buf = pktbuf_dynamic(256);
+	if (!buf) {
+		admin_error(admin, "no mem");
+		return true;
+	}
+	pktbuf_write_RowDescription(buf, "sqs", "hostname", "ttl", "addrs");
+	adns_walk_names(adns, dns_name_cb, buf);
+	admin_flush(admin, buf, "SHOW");
+	return true;
+}
+
+/* Command: SHOW DNS_ZONES */
+
+static void dns_zone_cb(void *arg, const char *name, uint32_t serial, int nhosts)
+{
+	PktBuf *buf = arg;
+	pktbuf_write_DataRow(buf, "sqi", name, (uint64_t)serial, nhosts);
+}
+
+static bool admin_show_dns_zones(PgSocket *admin, const char *arg)
+{
+	PktBuf *buf;
+
+	buf = pktbuf_dynamic(256);
+	if (!buf) {
+		admin_error(admin, "no mem");
+		return true;
+	}
+	pktbuf_write_RowDescription(buf, "sqi", "zonename", "serial", "count");
+	adns_walk_zones(adns, dns_zone_cb, buf);
+	admin_flush(admin, buf, "SHOW");
+	return true;
+}
+
+/* Command: SHOW CONFIG */
+
 static void show_one_param(void *arg, const char *name, const char *val, bool reloadable)
 {
 	PktBuf *buf = arg;
@@ -755,7 +825,6 @@ static void show_one_param(void *arg, const char *name, const char *val, bool re
 			     reloadable ? "yes" : "no");
 }
 
-/* Command: SHOW CONFIG */
 static bool admin_show_config(PgSocket *admin, const char *arg)
 {
 	PktBuf *buf;
@@ -838,7 +907,7 @@ static bool admin_cmd_resume(PgSocket *admin, const char *arg)
 			return admin_error(admin, "Pooler is not paused/suspended");
 	} else {
 		PgDatabase *db = find_database(arg);
-		log_info("PAUSE '%s' command issued", arg);
+		log_info("RESUME '%s' command issued", arg);
 		if (db == NULL)
 			return admin_error(admin, "no such database: %s", arg);
 		if (!db->db_paused)
@@ -891,8 +960,14 @@ static bool admin_cmd_pause(PgSocket *admin, const char *arg)
 		PgDatabase *db;
 		log_info("PAUSE '%s' command issued", arg);
 		db = find_database(arg);
-		if (db == NULL)
-			return admin_error(admin, "no such database: %s", arg);
+		if (db == NULL) {
+			db = register_auto_database(arg);
+			if (db == NULL) {
+				return admin_error(admin, "no such database: %s", arg);
+			} else {
+				slog_info(admin, "registered new auto-database for PAUSE: %s", arg);
+			}
+		}
 		if (db == admin->pool->db)
 			return admin_error(admin, "cannot pause admin db: %s", arg);
 		db->db_paused = 1;
@@ -905,15 +980,66 @@ static bool admin_cmd_pause(PgSocket *admin, const char *arg)
 	return true;
 }
 
+/* Command: KILL */
+static bool admin_cmd_kill(PgSocket *admin, const char *arg)
+{
+	struct List *item, *tmp;
+	PgDatabase *db;
+	PgPool *pool;
+
+	if (!admin->admin_user)
+		return admin_error(admin, "admin access needed");
+
+	if (cf_pause_mode)
+		return admin_error(admin, "already suspended/paused");
+
+	if (!arg[0])
+		return admin_error(admin, "a database is required");
+
+	log_info("KILL '%s' command issued", arg);
+	db = find_database(arg);
+	if (db == NULL) {
+		db = register_auto_database(arg);
+		if (db == NULL) {
+			return admin_error(admin, "no such database: %s", arg);
+		} else {
+			slog_info(admin, "registered new auto-database for KILL: %s", arg);
+		}
+	}
+	if (db == admin->pool->db)
+		return admin_error(admin, "cannot kill admin db: %s", arg);
+
+	db->db_paused = 1;
+	statlist_for_each_safe(item, &pool_list, tmp) {
+		pool = container_of(item, PgPool, head);
+		if (pool->db == db)
+			kill_pool(pool);
+	}
+
+	return admin_ready(admin, "KILL");
+}
+
 /* extract substring from regex group */
 static void copy_arg(const char *src, regmatch_t *glist,
 		     int gnum, char *dst, unsigned dstmax)
 {
 	regmatch_t *g = &glist[gnum];
 	unsigned len = g->rm_eo - g->rm_so;
-	if (len < dstmax)
-		memcpy(dst, src + g->rm_so, len);
-	else
+	const char *s = src + g->rm_so;
+	char *d = dst;
+	unsigned i;
+	if (len < dstmax) {
+		if (*s == '"') {
+			for (i = 1; i < len - 1; i++) {
+				if (s[i] == '"' && s[i+1] == '"')
+					i++;
+				*d++ = s[i];
+			}
+			len = d - dst;
+		} else {
+			memcpy(dst, s, len);
+		}
+	} else
 		len = 0;
 	dst[len] = 0;
 }
@@ -948,11 +1074,13 @@ static bool admin_show_help(PgSocket *admin, const char *arg)
 		"D\n\tSHOW HELP|CONFIG|DATABASES"
 		"|POOLS|CLIENTS|SERVERS|VERSION\n"
 		"\tSHOW STATS|FDS|SOCKETS|ACTIVE_SOCKETS|LISTS|MEM\n"
+		"\tSHOW DNS_HOSTS|DNS_ZONES\n"
 		"\tSET key = arg\n"
 		"\tRELOAD\n"
 		"\tPAUSE [<db>]\n"
-		"\tSUSPEND\n"
 		"\tRESUME [<db>]\n"
+		"\tKILL <db>\n"
+		"\tSUSPEND\n"
 		"\tSHUTDOWN", "");
 	if (res)
 		res = admin_ready(admin, "SHOW");
@@ -997,6 +1125,8 @@ static struct cmd_lookup show_map [] = {
 	{"version", admin_show_version},
 	{"totals", admin_show_totals},
 	{"mem", admin_show_mem},
+	{"dns_hosts", admin_show_dns_hosts},
+	{"dns_zones", admin_show_dns_zones},
 	{NULL, NULL}
 };
 
@@ -1008,6 +1138,7 @@ static bool admin_cmd_show(PgSocket *admin, const char *arg)
 }
 
 static struct cmd_lookup cmd_list [] = {
+	{"kill", admin_cmd_kill},
 	{"pause", admin_cmd_pause},
 	{"reload", admin_cmd_reload},
 	{"resume", admin_cmd_resume},
