@@ -28,8 +28,8 @@
 int log_socket_prefix(enum LogLevel lev, void *ctx, char *dst, unsigned int dstlen)
 {
 	const struct PgSocket *sock = ctx;
-	char *user, *db, *host;
-	char host6[INET6_ADDRSTRLEN];
+	const char *user, *db, *host;
+	char host6[PGADDR_BUF];
 	int port;
 
 	/* no prefix */
@@ -42,8 +42,7 @@ int log_socket_prefix(enum LogLevel lev, void *ctx, char *dst, unsigned int dstl
 	if (pga_is_unix(&sock->remote_addr)) {
 		host = "unix";
 	} else {
-		pga_ntop(&sock->remote_addr, host6, INET6_ADDRSTRLEN);
-		host = host6;
+		host = pga_ntop(&sock->remote_addr, host6, sizeof(host6));
 	}
 	port = pga_port(&sock->remote_addr);
 
@@ -346,31 +345,62 @@ static inline unsigned pga_family(const PgAddr *a)
 	return a->sa.sa_family;
 }
 
+int pga_cmp_addr(const PgAddr *a, const PgAddr *b)
+{
+    if (pga_family(a) != pga_family(b))
+		return pga_family(a) - pga_family(b);
+
+	switch (pga_family(a)) {
+	case AF_INET:
+		return memcmp(&a->sin.sin_addr, &b->sin.sin_addr, sizeof(a->sin.sin_addr));
+		break;
+	case AF_INET6:
+		return memcmp(&a->sin6.sin6_addr, &b->sin6.sin6_addr, sizeof(a->sin6.sin6_addr));
+		break;
+	default:
+		log_error("pga_cmp_addr: unsupported family");
+		return 0;
+	}
+}
+
 /* convert pgaddr to string */
 const char *pga_ntop(const PgAddr *a, char *dst, int dstlen)
 {
+	const char *res = NULL;
+	char buf[PGADDR_BUF];
+
+	memset(buf, 0, sizeof(buf));
+
 	switch (pga_family(a)) {
 	case AF_UNIX:
-		strlcpy(dst, "unix", dstlen);
-		return dst;
+		res = "unix";
+		break;
 	case AF_INET:
-		return inet_ntop(AF_INET, &a->sin.sin_addr, dst, dstlen);
+		res = inet_ntop(AF_INET, &a->sin.sin_addr, buf, sizeof(buf));
+		break;
 	case AF_INET6:
-		return inet_ntop(AF_INET6, &a->sin6.sin6_addr, dst, dstlen);
+		res = inet_ntop(AF_INET6, &a->sin6.sin6_addr, buf, sizeof(buf));
+		break;
 	default:
-		return NULL;
+		res = "(bad-af)";
 	}
+	if (res == NULL)
+		res = "(err-ntop)";
+
+	strlcpy(dst, res, dstlen);
+	return dst;
 }
 
 /* parse address from string */
 bool pga_pton(PgAddr *a, const char *s, int port)
 {
-	int res;
+	int res = 1;
 	if (strcmp(s, "unix") == 0) {
 		pga_set(a, AF_UNIX, port);
-		return true;
-	}
-	if (strchr(s, ':')) {
+	} else if (strcmp(s, "*") == 0) {
+		pga_set(a, AF_INET, port);
+		a->sin.sin_addr.s_addr = htonl(INADDR_ANY);
+	} else if (strchr(s, ':')) {
 		pga_set(a, AF_INET6, port);
 		res = inet_pton(AF_INET6, s, &a->sin6.sin6_addr);
 	} else {
@@ -385,8 +415,7 @@ bool pga_pton(PgAddr *a, const char *s, int port)
 const char *pga_str(const PgAddr *a, char *dst, int dstlen)
 {
 	char buf[PGADDR_BUF];
-	if (!pga_ntop(a, buf, sizeof(buf)))
-		return NULL;
+	pga_ntop(a, buf, sizeof(buf));
 	snprintf(dst, dstlen, "%s@%d", buf, pga_port(a));
 	return dst;
 }
