@@ -8,6 +8,11 @@
 
 #include "system.h"
 
+#ifdef WIN32
+#undef strerror
+#undef main
+#endif
+
 #include <getopt.h>
 #include <event.h>
 #include <libpq-fe.h>
@@ -15,9 +20,12 @@
 static void log_error(const char *, ...);
 static void log_debug(const char *, ...);
 static void fatal(const char *fmt, ...);
+static void fatal_noexit(const char *fmt, ...);
 
 #include "list.h"
 
+static char *simple_query = "select 1";
+
 typedef void (*libev_cb_f)(int sock, short flags, void *arg);
 
 typedef struct DbConn {
@@ -118,6 +126,16 @@ static void fatal_perror(const char *err)
 	exit(1);
 }
 
+static void fatal_noexit(const char *fmt, ...)
+{
+	va_list ap;
+	char buf[1024];
+	va_start(ap, fmt);
+	vsnprintf(buf, sizeof(buf), fmt, ap);
+	va_end(ap);
+	printf("FATAL: %s\n", buf);
+}
+
 static void fatal(const char *fmt, ...)
 {
 	va_list ap;
@@ -302,7 +320,7 @@ static int send_query_sleep(DbConn *db)
 
 static int send_query_simple(DbConn *db)
 {
-	const char *q = "select 1";
+	const char *q = simple_query;
 	return PQsendQueryParams(db->con, q, 0, NULL, NULL, NULL, NULL, 0);
 }
 
@@ -486,6 +504,7 @@ static const char usage_str [] =
 "  -C maxcps		max number of connects per sec\n"
 "  -Q maxqps		max number of queries per sec\n"
 "  -q num		queries per connection (default 1)\n"
+"  -S sql		set simple query\n"
 "accepted query types:\n"
 "  B - bigdata\n"
 "  S - sleep occasionally\n"
@@ -498,13 +517,20 @@ int main(int argc, char *argv[])
 	unsigned seed = time(NULL) ^ getpid();
 	char *cstr = NULL;
 	int numcon = 50;
+#ifdef WIN32
+	int wsresult;
+	WSADATA wsaData;
+#endif
 
-	while ((c = getopt(argc, argv, "d:n:s:t:hvC:Q:q:")) != EOF) {
+	while ((c = getopt(argc, argv, "S:d:n:s:t:hvC:Q:q:")) != EOF) {
 		switch (c) {
 		default:
 		case 'h':
 			printf("%s", usage_str);
 			return 0;
+		case 'S':
+			simple_query = optarg;
+			break;
 		case 'd':
 			cstr = optarg;
 			break;
@@ -542,7 +568,13 @@ int main(int argc, char *argv[])
 		printf(usage_str);
 		return 1;
 	}
-
+#ifdef WIN32
+        wsresult = WSAStartup(MAKEWORD(2,0),&wsaData);
+        if (wsresult != 0)
+        {
+                fatal("Cannot start the network subsystem -%d", wsresult);
+        }
+#endif
 	if (throttle_connects < 0 || throttle_queries < 0 || numcon < 0)
 		fatal("invalid parameter");
 
